import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { CatchClauseError } from './CatchClauseError.js';
import { Block } from './Block.js';
export class CatchClause {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.CatchClause;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        let i = 0;
        if (ast.error) {
            this.error = new CatchClauseError(ast.error, offsets[i], options);
            i += 1;
        }
        this.body = new Block(ast.body, offsets[i], options);
        metadata = updateMetadata(metadata, [this.error, this.body]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return ['catch ', path.call(print, 'error'), path.call(print, 'body')];
    }
}
//# sourceMappingURL=CatchClause.js.map