import { doc } from 'prettier';
import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { Identifier } from './Identifier.js';
import { ParametersDeclaration } from './ParametersDeclaration.js';
const { group } = doc.builders;
export class CatchClauseError {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.CatchClauseError;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        let i = 0;
        if (ast.name) {
            this.name = new Identifier(ast.name, offsets[i]);
            i += 1;
        }
        this.parameters = new ParametersDeclaration(ast.parameters, offsets[i], options);
        metadata = updateMetadata(metadata, [this.parameters]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return [
            path.call(print, 'name'),
            group(path.call(print, 'parameters')),
            ' '
        ];
    }
}
//# sourceMappingURL=CatchClauseError.js.map