import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { TypeName } from './TypeName.js';
import { Identifier } from './Identifier.js';
import { Expression } from './Expression.js';
export class ConstantDefinition {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.ConstantDefinition;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        this.typeName = new TypeName(ast.typeName, offsets[0], options);
        this.name = new Identifier(ast.name, offsets[1]);
        this.value = new Expression(ast.value, offsets[2], options);
        metadata = updateMetadata(metadata, [this.typeName, this.value]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return [
            path.call(print, 'typeName'),
            ' constant ',
            path.call(print, 'name'),
            ' = ',
            path.call(print, 'value'),
            ';'
        ];
    }
}
//# sourceMappingURL=ConstantDefinition.js.map