import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { UsingDirective } from './UsingDirective.js';
import { FunctionDefinition } from './FunctionDefinition.js';
import { ConstructorDefinition } from './ConstructorDefinition.js';
import { ReceiveFunctionDefinition } from './ReceiveFunctionDefinition.js';
import { FallbackFunctionDefinition } from './FallbackFunctionDefinition.js';
import { UnnamedFunctionDefinition } from './UnnamedFunctionDefinition.js';
import { ModifierDefinition } from './ModifierDefinition.js';
import { StructDefinition } from './StructDefinition.js';
import { EnumDefinition } from './EnumDefinition.js';
import { EventDefinition } from './EventDefinition.js';
import { StateVariableDefinition } from './StateVariableDefinition.js';
import { ErrorDefinition } from './ErrorDefinition.js';
import { UserDefinedValueTypeDefinition } from './UserDefinedValueTypeDefinition.js';
export class ContractMember {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.ContractMember;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        switch (ast.variant.cst.kind) {
            case NonterminalKind.UsingDirective:
                this.variant = new UsingDirective(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.FunctionDefinition:
                this.variant = new FunctionDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.ConstructorDefinition:
                this.variant = new ConstructorDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.ReceiveFunctionDefinition:
                this.variant = new ReceiveFunctionDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.FallbackFunctionDefinition:
                this.variant = new FallbackFunctionDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.UnnamedFunctionDefinition:
                this.variant = new UnnamedFunctionDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.ModifierDefinition:
                this.variant = new ModifierDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.StructDefinition:
                this.variant = new StructDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.EnumDefinition:
                this.variant = new EnumDefinition(ast.variant, offsets[0]);
                break;
            case NonterminalKind.EventDefinition:
                this.variant = new EventDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.StateVariableDefinition:
                this.variant = new StateVariableDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.ErrorDefinition:
                this.variant = new ErrorDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.UserDefinedValueTypeDefinition:
                this.variant = new UserDefinedValueTypeDefinition(ast.variant, offsets[0]);
                break;
            default:
                throw new Error(`Unexpected variant: ${ast.variant.cst.kind}`);
        }
        metadata = updateMetadata(metadata, [this.variant]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return path.call(print, 'variant');
    }
}
//# sourceMappingURL=ContractMember.js.map