import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { printSeparatedList } from '../slang-printers/print-separated-list.js';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { ErrorParameter } from './ErrorParameter.js';
export class ErrorParameters {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.ErrorParameters;
        let metadata = getNodeMetadata(ast, offset, true);
        const { offsets } = metadata;
        this.items = ast.items.map((item, index) => new ErrorParameter(item, offsets[index], options));
        this.separators = ast.separators.map((separator) => separator.unparse());
        metadata = updateMetadata(metadata, [this.items]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return this.items.length > 0
            ? printSeparatedList(path.map(print, 'items'))
            : '';
    }
}
//# sourceMappingURL=ErrorParameters.js.map