import { doc } from 'prettier';
import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { printSeparatedList } from '../slang-printers/print-separated-list.js';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { InheritanceType } from './InheritanceType.js';
const { line } = doc.builders;
export class InheritanceTypes {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.InheritanceTypes;
        let metadata = getNodeMetadata(ast, offset, true);
        const { offsets } = metadata;
        this.items = ast.items.map((item, index) => new InheritanceType(item, offsets[index], options));
        this.separators = ast.separators.map((separator) => separator.unparse());
        metadata = updateMetadata(metadata, [this.items]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return printSeparatedList(path.map(print, 'items'), {
            firstSeparator: line
        });
    }
}
//# sourceMappingURL=InheritanceTypes.js.map