import { doc } from 'prettier';
import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { Identifier } from './Identifier.js';
import { InheritanceSpecifier } from './InheritanceSpecifier.js';
import { InterfaceMembers } from './InterfaceMembers.js';
const { group, line } = doc.builders;
export class InterfaceDefinition {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.InterfaceDefinition;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        this.name = new Identifier(ast.name, offsets[0]);
        let i = 1;
        if (ast.inheritance) {
            this.inheritance = new InheritanceSpecifier(ast.inheritance, offsets[i], options);
            i += 1;
        }
        this.members = new InterfaceMembers(ast.members, offsets[i], options);
        metadata = updateMetadata(metadata, [this.inheritance, this.members]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return [
            group([
                'interface ',
                path.call(print, 'name'),
                this.inheritance ? [' ', path.call(print, 'inheritance')] : line,
                '{'
            ]),
            path.call(print, 'members'),
            '}'
        ];
    }
}
//# sourceMappingURL=InterfaceDefinition.js.map