import { doc } from 'prettier';
import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { Identifier } from './Identifier.js';
import { LibraryMembers } from './LibraryMembers.js';
const { group, line } = doc.builders;
export class LibraryDefinition {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.LibraryDefinition;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        this.name = new Identifier(ast.name, offsets[0]);
        this.members = new LibraryMembers(ast.members, offsets[1], options);
        metadata = updateMetadata(metadata, [this.members]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return [
            group(['library ', path.call(print, 'name'), line, '{']),
            path.call(print, 'members'),
            '}'
        ];
    }
}
//# sourceMappingURL=LibraryDefinition.js.map