import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { printLogicalOperation } from '../slang-printers/print-logical-operation.js';
import { createHugFunction } from '../slang-utils/create-hug-function.js';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { Expression } from './Expression.js';
const tryToHug = createHugFunction(['&&']);
export class OrExpression {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.OrExpression;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        this.leftOperand = new Expression(ast.leftOperand, offsets[0], options);
        this.operator = ast.operator.unparse();
        this.rightOperand = new Expression(ast.rightOperand, offsets[1], options);
        metadata = updateMetadata(metadata, [this.leftOperand, this.rightOperand]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
        this.leftOperand = tryToHug(this.leftOperand);
        this.rightOperand = tryToHug(this.rightOperand);
    }
    print(path, print, options) {
        return printLogicalOperation(this, path, print, options);
    }
}
//# sourceMappingURL=OrExpression.js.map