import { NonterminalKind, TerminalNode } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { ModifierInvocation } from './ModifierInvocation.js';
import { OverrideSpecifier } from './OverrideSpecifier.js';
export class ReceiveFunctionAttribute {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.ReceiveFunctionAttribute;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        if (ast.variant instanceof TerminalNode) {
            this.variant = ast.variant.unparse();
        }
        else {
            switch (ast.variant.cst.kind) {
                case NonterminalKind.ModifierInvocation:
                    this.variant = new ModifierInvocation(ast.variant, offsets[0], options);
                    break;
                case NonterminalKind.OverrideSpecifier:
                    this.variant = new OverrideSpecifier(ast.variant, offsets[0]);
                    break;
                default:
                    throw new Error(`Unexpected variant: ${ast.variant.cst.kind}`);
            }
        }
        metadata = updateMetadata(metadata, typeof this.variant === 'string' ? [] : [this.variant]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return typeof this.variant === 'string'
            ? this.variant
            : path.call(print, 'variant');
    }
}
//# sourceMappingURL=ReceiveFunctionAttribute.js.map