import { doc } from 'prettier';
import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { Expression } from './Expression.js';
const { group, indent, line } = doc.builders;
function printExpression(node, path, print, options) {
    if (node.expression) {
        return typeof node.expression.variant !== 'string' &&
            (node.expression.variant.kind === NonterminalKind.TupleExpression ||
                (options.experimentalTernaries &&
                    node.expression.variant.kind ===
                        NonterminalKind.ConditionalExpression))
            ? [' ', path.call(print, 'expression')]
            : group(indent([line, path.call(print, 'expression')]));
    }
    return '';
}
export class ReturnStatement {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.ReturnStatement;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        if (ast.expression) {
            this.expression = new Expression(ast.expression, offsets[0], options);
        }
        metadata = updateMetadata(metadata, [this.expression]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print, options) {
        return ['return', printExpression(this, path, print, options), ';'];
    }
}
//# sourceMappingURL=ReturnStatement.js.map