import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { joinExisting } from '../slang-utils/join-existing.js';
import { IdentifierPath } from './IdentifierPath.js';
import { ArgumentsDeclaration } from './ArgumentsDeclaration.js';
export class RevertStatement {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.RevertStatement;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        let i = 0;
        if (ast.error) {
            this.error = new IdentifierPath(ast.error, offsets[i]);
            i += 1;
        }
        this.arguments = new ArgumentsDeclaration(ast.arguments, offsets[i], options);
        metadata = updateMetadata(metadata, [this.error, this.arguments]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return [
            joinExisting(' ', ['revert', path.call(print, 'error')]),
            path.call(print, 'arguments'),
            ';'
        ];
    }
}
//# sourceMappingURL=RevertStatement.js.map