import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { PragmaDirective } from './PragmaDirective.js';
import { ImportDirective } from './ImportDirective.js';
import { ContractDefinition } from './ContractDefinition.js';
import { InterfaceDefinition } from './InterfaceDefinition.js';
import { LibraryDefinition } from './LibraryDefinition.js';
import { StructDefinition } from './StructDefinition.js';
import { EnumDefinition } from './EnumDefinition.js';
import { FunctionDefinition } from './FunctionDefinition.js';
import { ConstantDefinition } from './ConstantDefinition.js';
import { ErrorDefinition } from './ErrorDefinition.js';
import { UserDefinedValueTypeDefinition } from './UserDefinedValueTypeDefinition.js';
import { UsingDirective } from './UsingDirective.js';
import { EventDefinition } from './EventDefinition.js';
export class SourceUnitMember {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.SourceUnitMember;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        switch (ast.variant.cst.kind) {
            case NonterminalKind.PragmaDirective:
                this.variant = new PragmaDirective(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.ImportDirective:
                this.variant = new ImportDirective(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.ContractDefinition:
                this.variant = new ContractDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.InterfaceDefinition:
                this.variant = new InterfaceDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.LibraryDefinition:
                this.variant = new LibraryDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.StructDefinition:
                this.variant = new StructDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.EnumDefinition:
                this.variant = new EnumDefinition(ast.variant, offsets[0]);
                break;
            case NonterminalKind.FunctionDefinition:
                this.variant = new FunctionDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.ConstantDefinition:
                this.variant = new ConstantDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.ErrorDefinition:
                this.variant = new ErrorDefinition(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.UserDefinedValueTypeDefinition:
                this.variant = new UserDefinedValueTypeDefinition(ast.variant, offsets[0]);
                break;
            case NonterminalKind.UsingDirective:
                this.variant = new UsingDirective(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.EventDefinition:
                this.variant = new EventDefinition(ast.variant, offsets[0], options);
                break;
            default:
                throw new Error(`Unexpected variant: ${ast.variant.cst.kind}`);
        }
        metadata = updateMetadata(metadata, [this.variant]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return path.call(print, 'variant');
    }
}
//# sourceMappingURL=SourceUnitMember.js.map