import { doc } from 'prettier';
import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { printSeparatedItem } from '../slang-printers/print-separated-item.js';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { joinExisting } from '../slang-utils/join-existing.js';
import { Expression } from './Expression.js';
import { ReturnsDeclaration } from './ReturnsDeclaration.js';
import { Block } from './Block.js';
import { CatchClauses } from './CatchClauses.js';
const { line } = doc.builders;
export class TryStatement {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.TryStatement;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        this.expression = new Expression(ast.expression, offsets[0], options);
        let i = 1;
        if (ast.returns) {
            this.returns = new ReturnsDeclaration(ast.returns, offsets[i], options);
            i += 1;
        }
        this.body = new Block(ast.body, offsets[i], options);
        i += 1;
        this.catchClauses = new CatchClauses(ast.catchClauses, offsets[i], options);
        metadata = updateMetadata(metadata, [
            this.expression,
            this.returns,
            this.body,
            this.catchClauses
        ]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return [
            'try',
            printSeparatedItem(path.call(print, 'expression'), {
                firstSeparator: line
            }),
            joinExisting(' ', [
                path.call(print, 'returns'),
                path.call(print, 'body'),
                path.call(print, 'catchClauses')
            ])
        ];
    }
}
//# sourceMappingURL=TryStatement.js.map