import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { TypedTupleMember } from './TypedTupleMember.js';
import { UntypedTupleMember } from './UntypedTupleMember.js';
export class TupleMember {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.TupleMember;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        switch (ast.variant.cst.kind) {
            case NonterminalKind.TypedTupleMember:
                this.variant = new TypedTupleMember(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.UntypedTupleMember:
                this.variant = new UntypedTupleMember(ast.variant, offsets[0]);
                break;
            default:
                throw new Error(`Unexpected variant: ${ast.variant.cst.kind}`);
        }
        metadata = updateMetadata(metadata, [this.variant]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return path.call(print, 'variant');
    }
}
//# sourceMappingURL=TupleMember.js.map