import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { doc } from 'prettier';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { VariableDeclarationType } from './VariableDeclarationType.js';
import { StorageLocation } from './StorageLocation.js';
import { Identifier } from './Identifier.js';
import { VariableDeclarationValue } from './VariableDeclarationValue.js';
const { group, indent, indentIfBreak, line } = doc.builders;
export class VariableDeclarationStatement {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.VariableDeclarationStatement;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        this.variableType = new VariableDeclarationType(ast.variableType, offsets[0], options);
        let i = 1;
        if (ast.storageLocation) {
            this.storageLocation = new StorageLocation(ast.storageLocation, offsets[i]);
            i += 1;
        }
        this.name = new Identifier(ast.name, offsets[i]);
        i += 1;
        if (ast.value) {
            this.value = new VariableDeclarationValue(ast.value, offsets[i], options);
        }
        metadata = updateMetadata(metadata, [
            this.variableType,
            this.storageLocation,
            this.value
        ]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        const declarationDoc = group([
            path.call(print, 'variableType'),
            indent([
                this.storageLocation
                    ? [line, path.call(print, 'storageLocation')]
                    : '',
                ' ',
                path.call(print, 'name')
            ])
        ], { id: Symbol('Slang.VariableDeclarationStatement.variables') });
        return [
            declarationDoc,
            indentIfBreak(path.call(print, 'value'), { groupId: declarationDoc.id }),
            ';'
        ];
    }
}
//# sourceMappingURL=VariableDeclarationStatement.js.map