import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { YulFunctionCallExpression } from './YulFunctionCallExpression.js';
import { YulLiteral } from './YulLiteral.js';
import { YulBuiltInFunction } from './YulBuiltInFunction.js';
import { YulPath } from './YulPath.js';
export class YulExpression {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.YulExpression;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        switch (ast.variant.cst.kind) {
            case NonterminalKind.YulFunctionCallExpression:
                this.variant = new YulFunctionCallExpression(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.YulLiteral:
                this.variant = new YulLiteral(ast.variant, offsets[0], options);
                break;
            case NonterminalKind.YulBuiltInFunction:
                this.variant = new YulBuiltInFunction(ast.variant, offsets[0]);
                break;
            case NonterminalKind.YulPath:
                this.variant = new YulPath(ast.variant, offsets[0]);
                break;
            default:
                throw new Error(`Unexpected variant: ${ast.variant.cst.kind}`);
        }
        metadata = updateMetadata(metadata, [this.variant]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return path.call(print, 'variant');
    }
}
//# sourceMappingURL=YulExpression.js.map