import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { YulPathComponent } from './YulPathComponent.js';
export class YulPath {
    constructor(ast, offset) {
        this.kind = NonterminalKind.YulPath;
        let metadata = getNodeMetadata(ast, offset, true);
        const { offsets } = metadata;
        this.items = ast.items.map((item, index) => new YulPathComponent(item, offsets[index]));
        this.separators = ast.separators.map((separator) => separator.unparse());
        metadata = updateMetadata(metadata, [this.items]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return path
            .map(print, 'items')
            .map((item, index) => index === 0 ? item : [this.separators[index - 1], item]);
    }
}
//# sourceMappingURL=YulPath.js.map