import { NonterminalKind } from '@nomicfoundation/slang/cst';
import { getNodeMetadata, updateMetadata } from '../slang-utils/metadata.js';
import { joinExisting } from '../slang-utils/join-existing.js';
import { YulVariableDeclarationValue } from './YulVariableDeclarationValue.js';
import { YulVariableNames } from './YulVariableNames.js';
export class YulVariableDeclarationStatement {
    constructor(ast, offset, options) {
        this.kind = NonterminalKind.YulVariableDeclarationStatement;
        let metadata = getNodeMetadata(ast, offset);
        const { offsets } = metadata;
        this.variables = new YulVariableNames(ast.variables, offsets[0]);
        if (ast.value) {
            this.value = new YulVariableDeclarationValue(ast.value, offsets[1], options);
        }
        metadata = updateMetadata(metadata, [this.value]);
        this.comments = metadata.comments;
        this.loc = metadata.loc;
    }
    print(path, print) {
        return joinExisting(' ', [
            ['let', path.call(print, 'variables')],
            path.call(print, 'value')
        ]);
    }
}
//# sourceMappingURL=YulVariableDeclarationStatement.js.map